
from .giraffe_utils import GiraffeSetupInfo
from adari_core.report import AdariReportBase
from adari_core.plots.cut import CutPlot
from adari_core.plots.histogram import HistogramPlot
from adari_core.plots.images import ImagePlot, CentralImagePlot
from adari_core.plots.panel import Panel
from adari_core.plots.text import TextPlot
import adari_core.utils.clipping as clipping
from adari_core.utils.utils import fetch_kw_or_default
import os
import logging
logger = logging.getLogger(__name__)

class GiraffeFlatReport(AdariReportBase):
    category_label = ""
    center_size = 200
    def __init__(self):
        super().__init__("giraffe_flat")

    def parse_sof(self):
        master_flat = None
        raw_flat = None

        for filename, catg in self.inputs:
            if catg == "FIBER_FLAT" and raw_flat is None:
                raw_flat = filename
            elif catg == "MASTER_FIBER_FLAT" and master_flat is None:
                master_flat = filename
     

        file_lists = []
        if master_flat is not None and raw_flat is not None:
            file_lists.append(
                {
                    "master_im": master_flat,
                    "raw_im": raw_flat,
                }
            )
        return file_lists

   
    def generate_panels(self, **kwargs):
        """Create panels."""
        logger.info("Working on category {} report".format(self.category_label))
        # Retrieve appropriate metadata
        self.metadata = GiraffeSetupInfo.flat(list(self.hdus[0].values())[0])
        master_im = self.hdus[0]["master_im"]
        raw_im = self.hdus[0]["raw_im"]
        instru = fetch_kw_or_default(
            master_im["PRIMARY"], "INSTRUME", "Missing INSTRUME"
        )
        master_procatg = fetch_kw_or_default(
            master_im["PRIMARY"], "HIERARCH ESO PRO CATG", "Missing PRO CATG"
        )
        fname = os.path.basename(str(master_im.filename()))
        rawname = os.path.basename(str(raw_im.filename())).removeprefix("GIRAF.").removesuffix(".fits")
        panels = {}
        p = Panel(4, 3, height_ratios=[1, 4, 4])
        panels[p] = {
            "report_name": f"{self.name}_{rawname}_{master_procatg}",
            "report_description": "GIRAFFE flat - {}".format(
                self.category_label, fname
            ),
            "report_tags": [],
        }
        
        # Text Plot
        vspace = 0.3
        t1 = TextPlot(columns=1, v_space=vspace)
        col1 = (
            instru,
            "EXTNAME: "
            + str(fetch_kw_or_default(master_im["PRIMARY"], "EXTNAME", "N/A")),
            "PRO CATG: " + str(master_procatg),
            "FILE NAME: " + fname,
            "RAW1 NAME: "
            + str(
                fetch_kw_or_default(
                    master_im["PRIMARY"],
                    "HIERARCH ESO PRO REC1 RAW1 NAME",
                    "Missing RAW1 NAME",
                )
            ),
        )
        t1.add_data(col1)
        p.assign_plot(t1, 0, 0, xext=1)
        t2 = TextPlot(columns=1, v_space=vspace)
        col2 = self.metadata
        t2.add_data(col2)
        p.assign_plot(t2, 1, 0, xext=1)
        
        # Master Plots
        master_plot = ImagePlot(
            master_im["PRIMARY"].data, title=master_procatg
        )
        p.assign_plot(master_plot, 0, 1)
        
        # Get central image
        master_center = CentralImagePlot(
            master_plot,
            title=f"{master_procatg} " f"center",
            extent=self.center_size,
        )
        p.assign_plot(master_center, 0, 2)
        
        # Full raw flat
        full_raw = ImagePlot(
            title="Raw Flat",
            v_clip="minmax",
        )
        full_raw.add_data(raw_im["PRIMARY"].data)
        p.assign_plot(full_raw, 1, 1)
        
        # Get central image
        raw_center = CentralImagePlot(
            full_raw,
            title=f"Raw Flat center",
            extent=self.center_size,
        )
        p.assign_plot(raw_center, 1, 2)
        
        # Plot cut of master
        cutpos = master_plot.get_data_coord(master_plot.data.shape[0] // 2, "y")
        master_cut = CutPlot(
            "y",
            title="master column @ Y {}".format(cutpos),
            y_label=fetch_kw_or_default(
                master_im["PRIMARY"], "BUNIT", default="ADU"
            ),
        )
        master_cut.add_data(master_plot, cutpos, color="red", label="master")
        p.assign_plot(master_cut, 2, 1)
        
        cutpos = master_center.get_data_coord(master_center.data.shape[0] // 2, "y")
        master_cen_cut = CutPlot(
            "y",
            title="Central Region: column @ Y {}".format(cutpos),
            y_label=fetch_kw_or_default(
                master_im["PRIMARY"], "BUNIT", default="ADU"
            ),
        )
        master_cen_cut.add_data(master_center, cutpos, label="master", color="red")
        p.assign_plot(master_cen_cut, 2, 2)
        
        scaling = {}
        scaling["v_clip"] = "val"
        scaling["v_clip_kwargs"] = {"low": -5000, "high": 70000}
        
        raw_hist = HistogramPlot(
            raw_data=raw_im["PRIMARY"].data,
            title="raw value counts",
            bins=50,
            x_label=fetch_kw_or_default(raw_im["PRIMARY"], "BUNIT", "counts"),
            **scaling,
        )
        p.assign_plot(raw_hist, 3, 1)
        
        master_hist = HistogramPlot(
            master_data=master_plot.data,
            title="master value counts",
            bins=50,
            x_label=fetch_kw_or_default(
                master_im["PRIMARY"], "BUNIT", "counts"
            ),
            **scaling,
        )
        p.assign_plot(master_hist, 3, 2)
        return panels
        
rep = GiraffeFlatReport()
